SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, CONCAT_NULL_YIELDS_NULL, QUOTED_IDENTIFIER ON;
SET NUMERIC_ROUNDABORT OFF;
GO

IF (EXISTS (SELECT * 
                 FROM INFORMATION_SCHEMA.TABLES 
				 WHERE TABLE_SCHEMA = 'dbo' AND TABLE_NAME = 'aspnet_Membership'))
BEGIN
	IF NOT (EXISTS (SELECT * 
                 FROM INFORMATION_SCHEMA.TABLES 
				 WHERE TABLE_SCHEMA = 'dbo' AND TABLE_NAME = 'UserLogins'))
	BEGIN
		PRINT N'Creating table UserLogins...'
		CREATE TABLE [UserLogins] (
				[LoginProvider] NVARCHAR (128) NOT NULL,
				[ProviderKey]   NVARCHAR (128) NOT NULL,
				[UserId]        uniqueidentifier NOT NULL,
					CONSTRAINT [PK_dbo.UserLogins] PRIMARY KEY CLUSTERED ([LoginProvider] ASC, [ProviderKey] ASC, [UserId] ASC),
				CONSTRAINT [FK_dbo.UserLogins_dbo.aspnet_Users_UserId] FOREIGN KEY ([UserId]) REFERENCES [aspnet_Users] ([UserId]) ON DELETE CASCADE
			);
			CREATE NONCLUSTERED INDEX [IX_UserId]
				ON [UserLogins]([UserId] ASC);
	END
	ELSE
		PRINT N'The table UserLogins already present';

	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = 'PersistedGrants')
	BEGIN
		CREATE TABLE [PersistedGrants] (
			[Key] nvarchar(200) NOT NULL,
			[Type] nvarchar(50) NOT NULL,
			[SubjectId] nvarchar(200) NULL,
			[ClientId] nvarchar(200) NOT NULL,
			[CreationTime] datetime2 NOT NULL,
			[Expiration] datetime2 NULL,
			[Data] nvarchar(max) NOT NULL,
			[SessionId] nvarchar(100) NULL,
			[Description] nvarchar(200) NULL,
			[ConsumedTime] datetime2 NULL,
			CONSTRAINT [PK_PersistedGrants] PRIMARY KEY ([Key])
		);
	END
	ELSE
	BEGIN
		IF NOT EXISTS (
			SELECT * 
			FROM   sys.columns 
			WHERE  object_id = OBJECT_ID(N'[dbo].[PersistedGrants]') 
					AND name = 'SessionId'
		)
		BEGIN
			ALTER TABLE [PersistedGrants]
				ADD [SessionId] NVARCHAR (100) NULL
				PRINT N'Field [SessionId] added to [PersistedGrants] table';
		END

		IF NOT EXISTS (
			SELECT * 
			FROM   sys.columns 
			WHERE  object_id = OBJECT_ID(N'[dbo].[PersistedGrants]') 
					AND name = 'Description'
		)
		BEGIN
			ALTER TABLE [PersistedGrants]
				ADD [Description] NVARCHAR (200) NULL
				PRINT N'Field [Description] added to [PersistedGrants] table';
		END

		IF NOT EXISTS (
			SELECT * 
			FROM   sys.columns 
			WHERE  object_id = OBJECT_ID(N'[dbo].[PersistedGrants]') 
					AND name = 'ConsumedTime'
		)
		BEGIN
			ALTER TABLE [PersistedGrants]
				ADD [ConsumedTime] datetime2 NULL
				PRINT N'Field [ConsumedTime] added to [PersistedGrants] table';
		END
	END

	IF NOT EXISTS(
		SELECT * 
		FROM sys.indexes 
		WHERE object_id = OBJECT_ID(N'[dbo].[PersistedGrants]') 
			AND name = 'IX_PersistedGrants_SubjectId_SessionId_Type'
	)
	BEGIN
		CREATE INDEX [IX_PersistedGrants_SubjectId_SessionId_Type] ON [PersistedGrants] ([SubjectId] ASC, [SessionId] ASC,	[Type] ASC);
		PRINT N'Index [IX_PersistedGrants_SubjectId_SessionId_Type] created for [PersistedGrants] table';
	END

	IF NOT EXISTS(
		SELECT * 
		FROM sys.indexes 
		WHERE object_id = OBJECT_ID(N'[dbo].[PersistedGrants]') 
			AND name = 'IX_PersistedGrants_SubjectId_ClientId_Type'
	)
	BEGIN
		CREATE INDEX [IX_PersistedGrants_SubjectId_ClientId_Type] ON [PersistedGrants] ([SubjectId] ASC, [ClientId] ASC, [Type] ASC);
		PRINT N'Index [IX_PersistedGrants_SubjectId_ClientId_Type] created for [PersistedGrants] table';
	END		

	IF NOT EXISTS(
		SELECT * 
		FROM sys.indexes 
		WHERE object_id = OBJECT_ID(N'[dbo].[PersistedGrants]') 
			AND name = 'IX_PersistedGrants_Expiration'
	)
	BEGIN
		CREATE INDEX [IX_PersistedGrants_Expiration] ON [PersistedGrants] ([Expiration]);
		PRINT N'Index [IX_PersistedGrants_Expiration] created for [PersistedGrants] table';
	END		

	IF NOT EXISTS(
		SELECT * 
		FROM sys.indexes 
		WHERE object_id = OBJECT_ID(N'[dbo].[PersistedGrants]') 
			AND name = 'IX_PersistedGrants_ConsumedTime'
	)
	BEGIN
		CREATE INDEX [IX_PersistedGrants_ConsumedTime] ON [PersistedGrants] ([ConsumedTime]);
		PRINT N'Index [IX_PersistedGrants_ConsumedTime] created for [PersistedGrants] table';
	END

	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = 'DeviceCodes')
	BEGIN
		CREATE TABLE [DeviceCodes] (
			[UserCode] nvarchar(200) NOT NULL,
			[DeviceCode] nvarchar(200) NOT NULL,
			[SubjectId] nvarchar(200) NULL,
			[ClientId] nvarchar(200) NOT NULL,
			[CreationTime] datetime2 NOT NULL,
			[Expiration] datetime2 NOT NULL,
			[Data] nvarchar(max) NOT NULL,
			[SessionId] nvarchar(100) NULL,
			[Description] nvarchar(200) NULL,
			CONSTRAINT [PK_DeviceCodes] PRIMARY KEY ([UserCode])
		);
    
		CREATE UNIQUE INDEX [IX_DeviceCodes_DeviceCode] ON [DeviceCodes] ([DeviceCode]);
		CREATE INDEX [IX_DeviceCodes_Expiration] ON [DeviceCodes] ([Expiration]);
	END
	ELSE
	BEGIN
		IF NOT EXISTS (
			SELECT * 
			FROM   sys.columns 
			WHERE  object_id = OBJECT_ID(N'[dbo].[DeviceCodes]') 
					AND name = 'SessionId'
		)
		BEGIN
			ALTER TABLE [DeviceCodes]
				ADD [SessionId] NVARCHAR (100) NULL
				PRINT N'Field [SessionId] added to [DeviceCodes] table';
		END

		IF NOT EXISTS (
			SELECT * 
			FROM   sys.columns 
			WHERE  object_id = OBJECT_ID(N'[dbo].[DeviceCodes]') 
					AND name = 'Description'
		)
		BEGIN
			ALTER TABLE [DeviceCodes]
				ADD [Description] NVARCHAR (200) NULL
				PRINT N'Field [Description] added to [DeviceCodes] table';
		END
	END

	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = 'ExternalUserData')
	BEGIN
		CREATE TABLE [ExternalUserData] (
			[Key] int IDENTITY(1,1),
			[ProviderName] nvarchar(200) NOT NULL,
			[UserId] nvarchar(200) NOT NULL,
			[IsActive] bit NOT NULL,
			[Data] nvarchar(max) NOT NULL,
			CONSTRAINT [PK_ExternalUserData] PRIMARY KEY ([Key])
		);

		CREATE UNIQUE INDEX [IX_ExternalUserData_ProviderName_UserId] ON [ExternalUserData] ([ProviderName], [UserId]);
	END
	
	IF NOT EXISTS (SELECT * FROM [dbo].[aspnet_Roles] WHERE [RoleName] = 'sitecore\Sitecore XConnect Data Admin')
	BEGIN
		PRINT N'Creating Sitecore XConnect Data Admin Role...'
		DECLARE @ApplicationId nvarchar(256) = 
			(SELECT [ApplicationId] 
			FROM [dbo].[aspnet_Applications]
			WHERE [ApplicationName] = 'sitecore');
		DECLARE @RoleId uniqueIdentifier = NEWID();
		
		INSERT INTO [dbo].[aspnet_Roles] (
			[ApplicationId],
			[RoleId],
			[RoleName],
			[LoweredRoleName])
		VALUES (
			@ApplicationId,
			@RoleId,
			'sitecore\Sitecore XConnect Data Admin',
			'sitecore\sitecore xconnect data admin');
	END
END

GO

/* Authorize relevant roles permission for Marketing Automation */
DECLARE @SetMAEditorsAppID nvarchar(256) = (SELECT TOP(1) ApplicationId FROM aspnet_Applications WHERE ApplicationName = 'sitecore')
DECLARE @SetMAEditorsRoleID UNIQUEIDENTIFIER = NEWID()
DECLARE @MAEditorsName  varchar(100) = 'sitecore\Marketing Automation Editors'
DECLARE @AMRoleName varchar(100) = 'sitecore\Analytics Maintaining'
DECLARE @ACPRoleName varchar(100) = 'sitecore\Analytics Content Profiling'
DECLARE @AutRoleName varchar(100) = 'sitecore\Author'
DECLARE @LMERoleName varchar(100)  = 'sitecore\List Manager Editors'

IF NOT EXISTS (SELECT TOP(1)RoleId FROM aspnet_Roles WHERE roleName = @MAEditorsName )
BEGIN
	BEGIN TRAN
	
	INSERT INTO aspnet_Roles (ApplicationId,RoleId,RoleName,LoweredRoleName) 
	VALUES (@SetMAEditorsAppID, @SetMAEditorsRoleID, @MAEditorsName,LOWER(@MAEditorsName))
		
	INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@MAEditorsName,@AMRoleName,'',GETUTCDATE())
	INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@MAEditorsName,@ACPRoleName,'',GETUTCDATE())
	INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@MAEditorsName,@AutRoleName,'',GETUTCDATE())
	INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@MAEditorsName,@LMERoleName,'',GETUTCDATE())
	COMMIT TRAN
END

GO

/* Authorize relevant roles permission for FORM */
DECLARE @SitecoreAppID nvarchar(256) = 
			(SELECT [ApplicationId] 
			FROM [dbo].[aspnet_Applications]
			WHERE [ApplicationName] = 'sitecore');
			
DECLARE @SetFormDataAdminRoleID UNIQUEIDENTIFIER = NEWID()
DECLARE @SetFormPubRoleID UNIQUEIDENTIFIER = NEWID()
DECLARE @SetFormEdiRoleID UNIQUEIDENTIFIER = NEWID()

DECLARE @FormDataAdminName  varchar(100) = 'sitecore\Forms Data Administrator'
DECLARE @FormPubName  varchar(100) = 'sitecore\Forms Publisher'
DECLARE @FormEdiName  varchar(100) = 'sitecore\Forms Editor'

DECLARE @SCARoleName varchar(100) = 'sitecore\Sitecore Client Authoring'
DECLARE @AMRoleName varchar(100) = 'sitecore\Analytics Maintaining'

SET XACT_ABORT ON -- Turns on rollback if T-SQL statement raises a run-time error.

IF NOT EXISTS (SELECT TOP(1)RoleId FROM aspnet_Roles WHERE roleName = @FormDataAdminName )
BEGIN
	BEGIN TRAN
	
	INSERT INTO aspnet_Roles (ApplicationId,RoleId,RoleName,LoweredRoleName) 
	VALUES (@SitecoreAppID, @SetFormDataAdminRoleID,@FormDataAdminName,LOWER(@FormDataAdminName))
		
	INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName, ApplicationName, Created) VALUES (NEWID(),@FormDataAdminName,@SCARoleName,'',GETDATE())
	COMMIT TRAN
END


IF NOT EXISTS (SELECT TOP(1)RoleId FROM aspnet_Roles WHERE roleName = @FormPubName )
BEGIN
	BEGIN TRAN
	
	INSERT INTO aspnet_Roles (ApplicationId,RoleId,RoleName,LoweredRoleName) 
	VALUES (@SitecoreAppID, @SetFormPubRoleID, @FormPubName,LOWER(@FormPubName))
		
	INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName, ApplicationName, Created) VALUES (NEWID(),@FormPubName,@SCARoleName,'',GETDATE())
	COMMIT TRAN
END


IF NOT EXISTS (SELECT TOP(1)RoleId FROM aspnet_Roles WHERE roleName = @FormEdiName )
BEGIN
	BEGIN TRAN
	
	INSERT INTO aspnet_Roles (ApplicationId,RoleId,RoleName,LoweredRoleName) 
	VALUES (@SitecoreAppID, @SetFormEdiRoleID, @FormEdiName,LOWER(@FormEdiName))
		
	INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName, ApplicationName, Created) VALUES (NEWID(),@FormEdiName,@SCARoleName,'',GETDATE())
	INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName, ApplicationName, Created) VALUES (NEWID(),@FormEdiName,@AMRoleName,'',GETDATE())
	COMMIT TRAN
END

GO

/*Update relevant roles for Explorer*/
DECLARE @roleName varchar(100)  = 'sitecore\Experience Explorer'
DECLARE @roleId uniqueidentifier 

SELECT @roleId=roleid FROM aspnet_Roles WHERE roleName = @roleName

IF (EXISTS (SELECT * 
                 FROM INFORMATION_SCHEMA.TABLES 
                 WHERE TABLE_SCHEMA = 'dbo' 
                 AND  TABLE_NAME = 'aspnet_Roles'))
BEGIN
	DELETE FROM aspnet_Roles WHERE roleName = @roleName
END

IF (EXISTS (SELECT * 
                 FROM INFORMATION_SCHEMA.TABLES 
                 WHERE TABLE_SCHEMA = 'dbo' 
                 AND  TABLE_NAME = 'aspnet_Roles'))
BEGIN
	DELETE FROM aspnet_Roles WHERE roleName = @roleName
END

IF (EXISTS (SELECT * 
                 FROM INFORMATION_SCHEMA.TABLES 
                 WHERE TABLE_SCHEMA = 'dbo' 
                 AND  TABLE_NAME = 'RolesInRoles'))
BEGIN
	DELETE FROM RolesInRoles WHERE TargetRoleName =@roleName OR MemberRoleName = @roleName
END

IF (EXISTS (SELECT * 
                 FROM INFORMATION_SCHEMA.TABLES 
                 WHERE TABLE_SCHEMA = 'dbo' 
                 AND  TABLE_NAME = 'aspnet_UsersInRoles'))
BEGIN
	DELETE FROM aspnet_UsersInRoles WHERE roleid =@roleId
END

IF (EXISTS (SELECT * 
                 FROM INFORMATION_SCHEMA.TABLES 
                 WHERE TABLE_SCHEMA = 'dbo' 
                 AND  TABLE_NAME = 'aspnet_UsersInRoles'))
BEGIN
	DELETE FROM aspnet_UsersInRoles WHERE roleid =@roleId
END

GO

-- Adds/removes roles in roles.
-- Should run on Core or security DB.
-- This is a replacement for Sitecore.Shell.Upgrades.UpdateRolesPostStep.

IF NOT EXISTS (
		SELECT [Id]
		FROM [RolesInRoles]
		WHERE [MemberRoleName] = 'sitecore\Sitecore Client Bucket Management'
			AND [TargetRoleName] = 'sitecore\Sitecore Client Configuring'
		)
BEGIN
	INSERT INTO [RolesInRoles] (Id, MemberRoleName, TargetRoleName, ApplicationName, Created)
	VALUES (NEWID(), 'sitecore\Sitecore Client Bucket Management', 'sitecore\Sitecore Client Configuring', '', GETDATE());

	PRINT N'Role sitecore\Sitecore Client Configuring added to role sitecore\Sitecore Client Bucket Management';
END

IF NOT EXISTS (
		SELECT [Id]
		FROM [RolesInRoles]
		WHERE [MemberRoleName] = 'sitecore\Sitecore Client Bucket Management'
			AND [TargetRoleName] = 'sitecore\Sitecore Client Maintaining'
		)
BEGIN
	INSERT INTO [RolesInRoles] (Id, MemberRoleName, TargetRoleName, ApplicationName, Created)
	VALUES (NEWID(), 'sitecore\Sitecore Client Bucket Management', 'sitecore\Sitecore Client Maintaining', '', GETDATE());

	PRINT N'Role sitecore\Sitecore Client Maintaining added to role sitecore\Sitecore Client Bucket Management';
END

IF NOT EXISTS (
		SELECT [Id]
		FROM [RolesInRoles]
		WHERE [MemberRoleName] = 'sitecore\Sitecore Client Bucket Management'
			AND [TargetRoleName] = 'sitecore\Sitecore Client Developing'
		)
BEGIN
	INSERT INTO [RolesInRoles] (Id, MemberRoleName, TargetRoleName, ApplicationName, Created)
	VALUES (NEWID(), 'sitecore\Sitecore Client Bucket Management', 'sitecore\Sitecore Client Developing', '', GETDATE());

	PRINT N'Role sitecore\Sitecore Client Developing added to role sitecore\Sitecore Client Bucket Management';
END

GO

-- Manipulates user profile: changes user portrait.
-- Should run on Core or security DB.
-- This is a replacement for Sitecore.Shell.Upgrades.Bug118164PostStep.


DECLARE @newProperyNames NVARCHAR(max);
DECLARE @propIndex INT;
DECLARE @nextPropIndex INT;
DECLARE @newPortraitName NVARCHAR(30);
DECLARE @newPortraitNameLen INT;
DECLARE @portraitNameLenDifference INT;
DECLARE @UserCursor AS CURSOR;
DECLARE @portraitName NVARCHAR(max);
DECLARE @userId UNIQUEIDENTIFIER;
DECLARE @propertyNames NVARCHAR(max);
DECLARE @singlePropertyName NVARCHAR(max);
DECLARE @singlePropertyType NVARCHAR(10);
DECLARE @singlePropertyStart INT;
DECLARE @singlePropertyEnd INT;

SET @newPortraitName = 'office/16x16/default_user.png'
SET @newPortraitNameLen = LEN(@newPortraitName)
SET @singlePropertyType = ''
SET @singlePropertyStart = 0
SET @singlePropertyEnd = 0
SET @userCursor = CURSOR
FOR

SELECT CASE PATINDEX('%People/16x16/Astrologer.png%', PropertyValuesString)
		WHEN 0
			THEN 'people/16x16/user1.png'
		ELSE 'People/16x16/Astrologer.png'
		END, users.UserId, PropertyNames
FROM aspnet_Users users, aspnet_Profile PROFILE
WHERE users.UserId = PROFILE.UserId
	AND (
		(
			LoweredUserName = 'sitecore\admin'
			AND PropertyValuesString LIKE '%People/16x16/Astrologer.png%'
			)
		OR (
			LoweredUserName != 'sitecore\admin'
			AND PropertyValuesString LIKE '%people/16x16/user1.png%'
			)
		)

--Portraite to be replaced for sitecore\admin user:      People/16x16/Astrologer.png
--Portraite to be replaced for not sitecore\admin users: people/16x16/user1.png
--New portraite for both types of users:                 office/16x16/default_user.png
OPEN @userCursor;

FETCH NEXT
FROM @userCursor
INTO @portraitName, @userid, @propertyNames;

WHILE @@FETCH_STATUS = 0
BEGIN
	SET @propIndex = 1;
	SET @portraitNameLenDifference = 0;
	SET @newProperyNames = ''

	WHILE (@propIndex >= 1)
	BEGIN
		SET @nextPropIndex = charindex(':', @propertyNames, @propIndex) + 1;

		IF (@nextPropIndex <= 1)
			BREAK;

		SET @singlePropertyName = substring(@propertyNames, @propIndex, @nextPropIndex - @propIndex - 1)
		SET @propIndex = @nextPropIndex
		SET @nextPropIndex = charindex(':', @propertyNames, @propIndex) + 1;

		IF (@nextPropIndex <= 1)
			BREAK;

		SET @singlePropertyType = substring(@propertyNames, @propIndex, @nextPropIndex - @propIndex - 1)
		SET @propIndex = @nextPropIndex
		SET @nextPropIndex = charindex(':', @propertyNames, @propIndex) + 1;

		IF (@nextPropIndex <= 1)
			BREAK;

		SET @singlePropertyStart = cast(substring(@propertyNames, @propIndex, @nextPropIndex - @propIndex - 1) AS INT)
		SET @propIndex = @nextPropIndex
		SET @nextPropIndex = charindex(':', @propertyNames, @propIndex) + 1;

		IF (@nextPropIndex <= 1)
			BREAK;

		SET @singlePropertyEnd = cast(substring(@propertyNames, @propIndex, @nextPropIndex - @propIndex - 1) AS INT)
		SET @propIndex = @nextPropIndex

		IF (@singlePropertyName = 'Portrait')
		BEGIN
			SET @portraitNameLenDifference = @newPortraitNameLen - len(@portraitName)
			SET @singlePropertyEnd = @singlePropertyEnd + @portraitNameLenDifference
		END
		ELSE
		BEGIN
			IF (@singlePropertyType = 'S')
				SET @singlePropertyStart = @singlePropertyStart + @portraitNameLenDifference
		END

		SET @newProperyNames = @newProperyNames + @singlePropertyName + ':' + @singlePropertyType + ':' + cast(@singlePropertyStart AS NVARCHAR) + ':' + cast(@singlePropertyEnd AS NVARCHAR) + ':'
	END

	UPDATE aspnet_Profile
	SET PropertyNames = @newProperyNames, PropertyValuesString = REPLACE(cast(PropertyValuesString AS NVARCHAR(max)), @portraitName, @newPortraitName)
	WHERE UserId = @userId

	FETCH NEXT
	FROM @userCursor
	INTO @portraitName, @userid, @propertyNames;
END

GO

-- Removes Social component role definitions and their membership in default roles.
-- Should run on Core or security DB.
-- This is a replacement for Sitecore.Shell.Upgrades.SC9Update (Sitecore.Update).


DELETE
FROM RolesInRoles
WHERE MemberRoleName = 'sitecore\Author'
	AND TargetRoleName = 'sitecore\Sitecore Client Social Authoring'

DELETE
FROM aspnet_Roles
WHERE RoleName IN (
		'sitecore\Twitter Message Reviewer',
		'sitecore\Social Message Workflow Reviewer',
		'sitecore\Social Message Workflow Editor',
		'sitecore\Social Message Author',
		'sitecore\Social Marketer Message Reviewer',
		'sitecore\Sitecore Client Social Authoring',
		'sitecore\Facebook Message Reviewer'
		)

PRINT N'Social security has been removed.';

GO

/*
This script to perform update on the EXM roles. Updates include 
1. Add roles: sitecore\EXM Advanced Users, sitecore\EXM Users
2. Add RolesInRoles : sitecore\Analytics Maintaining, sitecore\Analytics Reporting, sitecore\List Manager Editors 
*/
DECLARE @AppID UNIQUEIDENTIFIER
DECLARE @AdminRoleName nvarchar(50) = 'sitecore\Sitecore Local Administrators'

DECLARE @ExmUserRoleID UNIQUEIDENTIFIER = NEWID()
DECLARE @ExmUserRoleName nvarchar(50) = 'sitecore\EXM Users'

DECLARE @ExmAdvanceUserRoleID UNIQUEIDENTIFIER = NEWID()
DECLARE @ExmAdvanceUserRoleName nvarchar(50) = 'sitecore\EXM Advanced Users'

DECLARE @AnalyticsMaintainingRoleName nvarchar(50) = 'sitecore\Analytics Maintaining'
DECLARE @AnalyticsReportingRoleName nvarchar(50) = 'sitecore\Analytics Reporting'
DECLARE @ListManagerEditorsRoleName nvarchar(50) = 'sitecore\List Manager Editors'

Select @AppID = ApplicationId from aspnet_Roles WHERE roleName = @AdminRoleName

IF @AppID IS NOT NULL
BEGIN

	IF NOT EXISTS (SELECT TOP(1)RoleId FROM aspnet_Roles WHERE roleName = @ExmUserRoleName )
	BEGIN
		Print 'Inserting sitecore\EXM Users.'

    BEGIN TRAN

		INSERT INTO aspnet_Roles (ApplicationId,RoleId,RoleName,LoweredRoleName) 
		VALUES (@AppID, @ExmUserRoleID,@ExmUserRoleName,LOWER(@ExmUserRoleName))
		
		INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@ExmUserRoleName,@AnalyticsMaintainingRoleName,'',GETUTCDATE())
		INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@ExmUserRoleName,@AnalyticsReportingRoleName,'',GETUTCDATE())
		INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@ExmUserRoleName,@ListManagerEditorsRoleName,'',GETUTCDATE())

	  COMMIT TRAN
	END

	IF NOT EXISTS (SELECT TOP(1)RoleId FROM aspnet_Roles WHERE roleName = @ExmAdvanceUserRoleName )
	BEGIN  
		Print 'Inserting sitecore\EXM Advanced Users.'

  	BEGIN TRAN

		INSERT INTO aspnet_Roles (ApplicationId,RoleId,RoleName,LoweredRoleName) 
		VALUES (@AppID, @ExmAdvanceUserRoleID,@ExmAdvanceUserRoleName,LOWER(@ExmAdvanceUserRoleName))
	
		INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@ExmAdvanceUserRoleName,@AnalyticsMaintainingRoleName,'',GETUTCDATE())
		INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@ExmAdvanceUserRoleName,@AnalyticsReportingRoleName,'',GETUTCDATE())
		INSERT INTO RolesInRoles(Id,MemberRoleName,TargetRoleName,ApplicationName,Created) VALUES (NEWID(),@ExmAdvanceUserRoleName,@ListManagerEditorsRoleName,'',GETUTCDATE())
  	
    COMMIT TRAN
	END
END
ELSE
BEGIN
Print 'Failed to update EXM. Application does not exists'
END

GO

PRINT N'Update complete.';