﻿<%@ Page Language="c#" Inherits="Sitecore.sitecore.admin.AdminPage, Sitecore.Client" CodePage="65001" Debug="true" Async="true" %>

<%@ Import Namespace="System.Globalization" %>
<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="System.Reflection" %>
<%@ Import Namespace="System.Threading.Tasks" %>
<%@ Import Namespace="Sitecore.Globalization" %>
<%@ Import Namespace="Sitecore.Data" %>
<%@ Import Namespace="Sitecore.Data.Items" %>
<%@ Import Namespace="Sitecore.Data.Managers" %>
<%@ Import Namespace="Sitecore.Marketing.Definitions" %>
<%@ Import Namespace="Sitecore.Marketing.Definitions.MarketingAssets" %>

<%@ OutputCache Location="None" VaryByParam="none" %>

<!DOCTYPE html>

<script runat="server">

    private const string FileMask = "RestoredDefinitions_{0}.dat";
    private const string AppDataPath = "~/App_Data/";
    private const string DefaultWorkflowId = "{689E2994-4656-4C58-9112-7826CB46EE69}";
    private const string AnalyticsWorkflowFinalStateId = "{EDCBB550-BED3-490F-82B8-7B2F14CCD26E}";
    private readonly ID _fileTemplateId = new ID("{962B53C4-F93B-4DF9-9821-415C867B8903}");

    private readonly string[] _definitionTypes =
        {
            "Sitecore.Marketing.Definitions.Campaigns.ICampaignActivityDefinition, Sitecore.Marketing",
            "Sitecore.Marketing.Definitions.Goals.IGoalDefinition, Sitecore.Marketing",
            "Sitecore.Marketing.Definitions.Outcomes.Model.IOutcomeDefinition, Sitecore.Marketing",
            "Sitecore.Marketing.Definitions.Funnels.IFunnelDefinition, Sitecore.Marketing",
            "Sitecore.Marketing.Definitions.MarketingAssets.IMarketingAssetDefinition, Sitecore.Marketing"
        };

    protected override void OnInit(EventArgs arguments)
    {
        CheckSecurity(true);
        Directory.CreateDirectory(Server.MapPath(AppDataPath));
        base.OnInit(arguments);
    }

    protected void Page_PreRender(object sender, EventArgs e)
    {
        FileRepeater.DataSource = Directory.EnumerateFiles(Server.MapPath(AppDataPath), string.Format(FileMask, "*")).ToArray();
        FileRepeater.DataBind();
    }

    protected void btnRemoveRestored_Click(object sender, CommandEventArgs e)
    {
        if (e.CommandArgument == null)
        {
            return;
        }

        spanRestored.InnerHtml = "";

        string fileName = e.CommandArgument.ToString();
        bool loadedSuccessfully;
        var itemsData = LoadData(fileName, out loadedSuccessfully);

        if (loadedSuccessfully && RemoveDefinitions(itemsData))
        {
            WriteLine("Deleting restored items - complete.");
            File.Delete(fileName);
            WriteLine(string.Format("File {0} was removed successfully.", fileName));
        }
    }

    protected async void btnFindAndRestoreDeletedDefinitions_Click(object sender, EventArgs e)
    {
        spanRestored.InnerHtml = "";
        await FindAndRestoreDeletedDefinitions();
    }

    private List<ItemData> LoadData(string fileName, out bool loadedSuccessfully)
    {
        var itemsData = new List<ItemData>();
        loadedSuccessfully = true;

        using (var tr = File.OpenText(fileName))
        {
            while (!tr.EndOfStream)
            {
                var line = tr.ReadLine();

                if (string.IsNullOrWhiteSpace(line))
                {
                    continue;
                }

                ItemData itemData;

                if (!TryParseItemLine(line, out itemData))
                {
                    loadedSuccessfully = false;
                    continue;
                }

                itemsData.Add(itemData);
            }
        }

        return itemsData;
    }

    private bool RemoveDefinitions(List<ItemData> itemsData)
    {
        var masterDatabase = Sitecore.Configuration.Factory.GetDatabase("master");
        bool isSuccess = true;

        foreach (ItemData itemData in itemsData)
        {
            try
            {
                switch (itemData.Operation)
                {
                    case ItemOperation.RestoredItem:
                        var itemToRemove = masterDatabase.GetItem(itemData.Id);

                        if (itemToRemove != null)
                        {
                            itemToRemove.Delete();
                            WriteLine("Deleted item: " + itemData.TypeName + ":" + itemData.Id);
                        }

                        break;
                    case ItemOperation.RestoredVersion:
                        var item = masterDatabase.GetItem(itemData.Id, itemData.Language, Sitecore.Data.Version.Parse(itemData.Version));

                        if (item != null)
                        {
                            item.Versions.RemoveVersion();

                            WriteLineFormat("Deleted version: {0}:{1} ({2}, version {3})", itemData.TypeName, itemData.Id, itemData.Language, itemData.Version);
                        }

                        break;
                }

            }
            catch (Exception ex)
            {
                WriteLineFormat("Error deleting: {0}:{1} ({2}, version {3}) <br/> {4} ", itemData.TypeName, itemData.Id, itemData.Language, itemData.Version, ex);
                isSuccess = false;
            }
        }

        return isSuccess;
    }

    private void WriteLineFormat(string format, params object[] parameters)
    {
        WriteLine(string.Format(format, parameters));
    }

    private void WriteLine(string line)
    {
        spanRestored.Visible = true;
        spanRestored.InnerHtml += line;
        spanRestored.InnerHtml += @"<br/>";
    }

    private bool TryParseItemLine(string line, out ItemData itemData)
    {
        try
        {
            var lineParts = line.Split('#');

            if (lineParts.Length != 2)
            {
                throw new InvalidOperationException("Invalid operation format.");
            }

            Sitecore.Data.Version version = null;
            Language language = null;
            ItemOperation operation;

            if (!Enum.TryParse(lineParts[0], true, out operation))
            {
                throw new InvalidOperationException("Invalid operation.");
            }

            lineParts = lineParts[1].Split(new[] { ',', ':', ' ' }, StringSplitOptions.RemoveEmptyEntries);


            if (operation == ItemOperation.RestoredItem)
            {
                if (lineParts.Length < 2)
                {
                    throw new InvalidOperationException("Not enough parameters for item remove operation.");
                }
            }

            if (operation == ItemOperation.RestoredVersion)
            {
                if (lineParts.Length < 6)
                {
                    throw new InvalidOperationException("Not enough parameters for item version remove operation.");
                }

                if (lineParts[2] != "Language"
                    || !Language.TryParse(lineParts[3], out language)
                    || lineParts[4] != "Version"
                    || !Sitecore.Data.Version.TryParse(lineParts[5], out version))
                {
                    throw new InvalidOperationException("Invalid format arguments");
                }
            }

            if (_definitionTypes.All(t => !t.Contains(lineParts[0])))
            {
                throw new InvalidOperationException("Unknown definition type " + lineParts[0]);
            }

            if (!Sitecore.Data.ID.IsID(lineParts[1]))
            {
                throw new InvalidOperationException("Invalid item id: " + lineParts[0]);
            }

            itemData = new ItemData
            {
                Operation = operation,
                TypeName = lineParts[0],
                Id = new ID(lineParts[1]),
                Language = language,
                Version = version
            };

        }
        catch (Exception exception)
        {

            WriteLineFormat("Error parsing format: \"{0}\" <BR/> {1}", line, exception);

            itemData = null;
            return false;
        }

        return true;
    }

    private async Task FindAndRestoreDeletedDefinitions()
    {
        var fileName = Server.MapPath(Path.Combine(AppDataPath, string.Format(FileMask, DateTime.UtcNow.ToString("yyyy_MM_dd_HH_mm_ss"))));

        using (TextWriter tw = TextWriter.Synchronized(File.CreateText(fileName)))
        {
            List<Task> tasks = new List<Task>();

            var method = GetType().GetMethod("FindAndRestoreDeletedDefinitionsAsync", BindingFlags.NonPublic | BindingFlags.Instance);
            System.Diagnostics.Debug.WriteLine(method);

            foreach (var definitionType in _definitionTypes)
            {
                var type = GetType(definitionType);

                if (type == null)
                {
                    continue;
                }

                var genericMethod = method.MakeGenericMethod(type);
                var task = genericMethod.Invoke(this, new object[] { tw }) as Task;

                if (task != null)
                {
                    tasks.Add(task);
                }
            }

            await Task.WhenAll(tasks.ToArray()).ConfigureAwait(false);
        }
    }

    private Task FindAndRestoreDeletedDefinitionsAsync<TDefinitionInterface>(TextWriter tw) where TDefinitionInterface : IDefinition
    {
        return Task.Factory.StartNew(async () =>
        {
            var restoredItemIds = new HashSet<ID>();
            var definitionList = GetAllDefinitions<TDefinitionInterface>();

            dynamic definitionManagerFactory = typeof(DefinitionManagerFactory).GetProperty("Default").GetValue(null);
            var masterDatabase = Sitecore.Configuration.Factory.GetDatabase("master");
            var targetManager = definitionManagerFactory.GetDefinitionManager<TDefinitionInterface>("item") as IDefinitionManager<TDefinitionInterface>;

            int numberOfRestoredItems = 0;

            Item mediaLibrary = null;

            if (typeof(TDefinitionInterface) == typeof(IMarketingAssetDefinition))
            {
                mediaLibrary = masterDatabase.GetItem(Sitecore.ItemIDs.MediaLibraryRoot);
            }

            foreach (var definition in definitionList)
            {
                ID definitionId = new ID(definition.Id.ToString());

                var version = Sitecore.Data.Version.Parse(definition.Version);
                var language = Language.Parse(definition.Culture.Name);
                var item = masterDatabase.GetItem(definitionId, language);

                var itemData = new ItemData
                {
                    Operation = ItemOperation.None,
                    Id = definitionId,
                    Version = version,
                    Language = language,
                    TypeName = typeof(TDefinitionInterface).Name
                };

                if (item == null)
                {
                    itemData.Operation = ItemOperation.RestoredItem;
                    restoredItemIds.Add(definitionId);
                    tw.WriteLine(itemData.ToString());
                }
                else
                {
                    if (!item.Versions.GetVersionNumbers().Contains(version))
                    {
                        itemData.Operation = ItemOperation.RestoredVersion;

                        if (!restoredItemIds.Contains(definitionId))
                        {
                            tw.WriteLine(itemData.ToString());
                        }
                    }
                }

                if (itemData.Operation != ItemOperation.None)
                {
                    if (typeof(TDefinitionInterface) == typeof(IMarketingAssetDefinition) && itemData.Operation == ItemOperation.RestoredItem)
                    {
                        var itemWithAsset = ItemManager.CreateItem(definition.Name, mediaLibrary, _fileTemplateId, definitionId);
                        itemWithAsset.Editing.BeginEdit();

                        foreach (var customValue in definition.CustomValues)
                        {
                            itemWithAsset.Fields[new ID(customValue.Key.ToString())].SetValue(customValue.Value, true);
                        }
                        itemWithAsset["B0A67B2A-8B07-4E0B-8809-69F751709806"] = "<tracking><event id=\"{FA72E131-3CFD-481C-8E15-04496E9586DC}\" name=\"Download\" /></tracking>";

                        itemWithAsset.Editing.EndEdit();
                    }

                    await targetManager.SaveAsync(definition);
                    numberOfRestoredItems++;

                    var savedItem = masterDatabase.GetItem(definitionId, itemData.Language, itemData.Version);
                    if (savedItem != null)
                    {

                        using (new EditContext(savedItem))
                        {
                            savedItem.Fields["__Workflow"].Value = DefaultWorkflowId;
                            savedItem.Fields["__Workflow state"].Value = AnalyticsWorkflowFinalStateId;
                        }
                    }
                }
            }

            WriteLineFormat("{0}: {1} items and item versions were restored", typeof(TDefinitionInterface).Name, numberOfRestoredItems);
            restoredItemIds.Clear();
        },
        TaskCreationOptions.LongRunning);
    }

    private static IEnumerable<TDefinitionInterface> GetAllDefinitions<TDefinitionInterface>() where TDefinitionInterface : IDefinition
    {
        dynamic definitionManagerFactory = typeof(DefinitionManagerFactory).GetProperty("Default").GetValue(null);
        var sourceManager = definitionManagerFactory.GetDefinitionManager<TDefinitionInterface>("rdb") as IDefinitionManager<TDefinitionInterface>;

        var parameters = new RetrievalParameters<TDefinitionInterface, string>(
            pageSize: int.MaxValue,
            includeLatestVersionOnly: true,
            includeResultsThatDoesNotMatchTargetCulture: true);

        var definitionResultSet = sourceManager.GetAll(
            CultureInfo.InvariantCulture,
            parameters,
            true);

        var cultures = definitionResultSet
            .SelectMany(d => sourceManager.GetAvailableCultures(d.Data.Id))
            .Distinct()
            .ToList();

        parameters = new RetrievalParameters<TDefinitionInterface, string>(
            pageSize: int.MaxValue,
            includeLatestVersionOnly: false,
            includeResultsThatDoesNotMatchTargetCulture: false);

        var definitionList = cultures
            .SelectMany(c =>
                sourceManager.GetAll(c, parameters, true)
                    .Select(d => d.Data)
                    .OrderBy(d => d.Version))
            .ToList();
        return definitionList;
    }

    private static Type GetType(string fullName)
    {
        return Type.GetType(fullName, false);
    }

    enum ItemOperation
    {
        None,
        RestoredItem,
        RestoredVersion
    }

    class ItemData
    {
        public ItemOperation Operation { get; set; }

        public ID Id { get; set; }

        public string TypeName { get; set; }

        public Sitecore.Data.Version Version { get; set; }

        public Language Language { get; set; }

        public override string ToString()
        {
            string line = string.Empty;

            switch (Operation)
            {
                case ItemOperation.RestoredItem:
                    line = Operation + "#" + TypeName + ":" + Id;
                    break;
                case ItemOperation.RestoredVersion:
                    line = Operation + "#" + TypeName + ":" + Id + ", Language: " + Language + ", Version: " + Version;
                    break;
            }

            return line;
        }
    }

</script>

<html lang="en">
<head>
    <title>Restore Deleted Marketing Definitions</title>
    <link rel="shortcut icon" href="/sitecore/images/favicon.ico" />
    <link href="/default.css" rel="stylesheet" />
    <style type="text/css">
        body {
            padding: 5px;
        }
        .note {
            background-color: lightgray;
        }
        #report {
            padding: 20px 0;
        }
    </style>
</head>
<body>
    <h1>Restore Deleted Marketing Definitions</h1>
    <p class="note">
        In Sitecore 9.0, marketing definitions are no longer deployed to the reporting database. Instead, they are deployed to the Reference Data service. As part of upgrade to 9.0 it is necessary to copy all marketing definitions from the reporting database to the Reference Data Service.
        <br/><br/>
        The easiest way to deploy all marketing definitions to the Reference Data service is to use the Deploy marketing definitions dialog (Control Panel -> Analytics -> Deploy marketing definitions) after Sitecore is upgraded to 9.0. This dialog gets all marketing definitions from the content database and deploys them to the Reference Data service.
        <br/><br/>
        However, there may be situations when a marketing definition exists in the reporting database but the corresponding marketing definition item no longer exists in the content database. This can happen when a marketing definition item is created in the content database and deployed to the reporting database, after which the original item is deleted. For example, your marketing team might remove campaigns that are no longer active.
        <br /><br />
        In previous versions of Sitecore, definitions were maintained in the reporting database even when corresponding items were removed from the content database. This was done by manually copying tables before rebuilding the reporting database, and was required to ensure that the rebuild did not fail when old interactions with references to deleted definitions were processed.
        <br/><br/>
        This page gets marketing definitions from the reporting database and restores corresponding items/versions if they no longer exist in the content database. These restored items will be automatically updated during the installation of upgrade package. Once upgrade package is installed, all marketing definitions have to be deployed to the Reference Data service with the help of Deploy marketing definitions dialog. 
        <br /><br />
        Once that is done, restored items/versions can be safely removed by using this page as it keeps track of restored items/versions. If you choose not to delete the restored items, make sure your marketing team is aware that previously deleted items were restored as part of the upgrade process.
        <br/><br/>
        <b>Important note:</b> This page uses <code>rdb</code> Marketing Operations repository, which means that the current Sitecore instance must have connection to the reporting database.
    </p>
    <form method="post" runat="server" id="mainform">
        <div id="MainPanel">
            <asp:Button ID="btnFindAndRestoreDeletedDefinitions" OnClick="btnFindAndRestoreDeletedDefinitions_Click" runat="server" Text="Restore deleted definition items" OnClientClick="this.disabled = true; this.value = 'Restoring. Please wait...';" UseSubmitBehavior="false" />
            <div id="report">
                <span runat="server" id="spanRestored" Visible="False"></span>
            </div>
            <asp:Repeater runat="server" ItemType="System.String" ID="FileRepeater">
                <HeaderTemplate>
                    <table>
                </HeaderTemplate>
                <ItemTemplate>
                    <tr>
                        <td><%# Item %> </td>
                        <td>
                            <asp:Button ID="btnRemoveRestored" CommandArgument='<%# Item %>' OnCommand="btnRemoveRestored_Click" runat="server" Text="Remove restored definition items" OnClientClick="this.disabled = true; this.value = 'Removing. Please wait...';" UseSubmitBehavior="false" />
                        </td>
                    </tr>
                </ItemTemplate>
                <FooterTemplate>
                    </table>
                </FooterTemplate>
            </asp:Repeater>

        </div>
    </form>
</body>
</html>
